/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.collections;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * 
 * @author Torgil
 *
 */
public class UniqueHashBag<K, V> {

    public static final int DEFAULT_MAP_INITIAL_CAPACITY = 16;
    public static final int DEFAULT_SET_INITIAL_CAPACITY = 10;
    
    private Map<K, Set<? extends V>>   m_hashBag;
    private int     m_initialSetCapacity;
    
    /**
     * Creates a <tt>HashBag</tt> with default initial capabilities.
     *
     */
    public UniqueHashBag() {
        this(DEFAULT_MAP_INITIAL_CAPACITY, DEFAULT_SET_INITIAL_CAPACITY);
    }
    
    /**
     * Creates a HashBag with the specified internal capacity of the internal
     * <tt>Map</tt>.
     * 
     * @param initialHashCapacity
     */
    public UniqueHashBag(int initialHashCapacity) {
        this(initialHashCapacity, DEFAULT_SET_INITIAL_CAPACITY);
    }
    
    /**
     * Creates a HashBag with the specified initial capacities of the internal
     * <tt>Map</tt> and the mapped <tt>List</tt>s.
     * 
     * @param initialHashCapacity
     * @param iInitialListCapacity
     */
    public UniqueHashBag(int initialHashCapacity, int iInitialListCapacity) {
        m_hashBag = new HashMap<K, Set<? extends V>>(initialHashCapacity);
        m_initialSetCapacity = iInitialListCapacity;
    }
    
    /**
     * 
     * @param key
     * @param value
     */
    @SuppressWarnings(value={"unchecked"})
    public void put(K key, V value) {
        Object o = m_hashBag.get(key);
        if (o instanceof Set) {
            ((Set<V>)o).add(value);
        }
        else {
            Set<V> set = createSet();
            set.add(value);
            m_hashBag.put(key, set);
        }
    }
    
    public void putSet(K key, Set<? extends V> value) {
        m_hashBag.put(key, value);
    }
    
    
    public void remove(K key) {
        m_hashBag.remove(key);
    }
    
    public void remove(K key, V value) {
        Set<? extends V> set = m_hashBag.get(key);
        if (set != null) {
            set.remove(value);
            if( set.isEmpty() )
                m_hashBag.remove(key);
        }
    }
    
    public Set<? extends V> get(K key) {
        return m_hashBag.get(key);
    }
    
    public Set<K> keySet() {
        return m_hashBag.keySet();
    }
    
    public Set entrySet() {
        return m_hashBag.entrySet();
    }
    
    /**
     * Returns the number of keys in the <tt>HashBag</tt>.
     * 
     */
    public int size() {
        return m_hashBag.size();
    }
    
    public boolean isEmpty() {
        return m_hashBag.isEmpty();
    }
    
    public boolean contains(K key) {
        return m_hashBag.containsKey(key);
    }
    
    public void clear() {
        m_hashBag.clear();
    }
    
    private Set<V> createSet() {
        return new HashSet<V>(m_initialSetCapacity);
    }
}
