/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml;

/**
 * An <code>IQualifiedName</code> is a pair consisting of a namespace prefix and
 * a local name. It represents the entity called "qualified name" in the 
 * WC3 document "Namespaces in XML 1.0":
 * <p>
 * http://www.w3.org/TR/xml-names/#concepts
 * 
 * @author Torgil
 * 
 * @since 1.2
 *
 */
public interface IQualifiedName {

    /**
     * Returns the namespace prefix of this <code>IQualifiedName</code>. An
     * <code>IQualifiedName</code> instance representing the name
     * <code>"tns:Root"</code> returns the string <code>"tns"</code> from
     * this method.
     * 
     * @return a String holding the prefix if this <code>IQualifiedName</code>.
     *         This method never returns <code>null</code>; an empty string
     *         is returned if this <code>IQualifiedName</code> is unprefixed.
     */
    public String getPrefix();
    
    /**
     * Returns the local part of this <code>IQualifiedName</code>. An
     * <code>IQualifiedName</code> instance representing the name
     * <code>"tns:Root"</code> returns the string <code>"Root"</code> from
     * this method.
     * 
     * @return a String holding the local part of this name. This method should
     *         never returns <code>null</code> or an empty String.
     */
    public String getLocalName();

    /**
     * Returns a string with the following format:
     * 
     * <pre>
     *    [getPrefix():]getLocalName()
     * </pre>
     * 
     * The prefix part is excluded if this name is unprefixed.
     * 
     */
    public String toString();
    
}
